@@ -1,5 +1,5 @@
 ;; -*- Mode: Lisp; Syntax: Common-Lisp -*-
-;; $Id: sockets.lisp,v 1.46 2009/03/30 12:41:11 jjgarcia Exp $
+;; $Id$
 
 ;; This file is based on SBCL's SB-BSD-SOCKET module and has been
 ;; heavily modified to work with ECL by Julian Stecklina.
@@ -50,7 +50,6 @@
  "#endif")
 #+:wsock
 (clines
- #+(or :msvc :mingw32)
  "#include <winsock2.h>"
  "typedef unsigned int uint32_t;"
  #-:mingw32
@@ -88,14 +87,14 @@
   (wsock-initialize)
 ); #+:wsock
 
-(eval-when (:compile-toplevel :load-toplevel)
+(eval-when (:compile-toplevel :execute)
+  (defmacro c-constant (c-name)
+    `(c-inline () () :int ,c-name :one-liner t))
   (defmacro define-c-constants (&rest args)
-    `(progn
+    `(let () ; Prevents evaluation of constant value form
        ,@(loop
 	    for (lisp-name c-name) on args by #'cddr
-	    collect `(defconstant ,lisp-name (c-inline () () :int ,c-name :one-liner t)))))
-  (defmacro c-constant (name)
-    `(c-inline () () :int ,name :one-liner t)))
+	    collect `(defconstant ,lisp-name (c-constant ,c-name))))))
 
 #+:wsock
 (Clines
@@ -158,7 +157,7 @@ containing the whole rest of the given `string', if any."
 (defmethod host-ent-address ((host-ent host-ent))
   (car (host-ent-addresses host-ent)))
 
-;; FIXME: Do we need to free the hostent structures?
+;; FIXME: We should move this to using getaddrinfo
 (defun get-host-by-name (host-name)
   "Returns a HOST-ENT instance for HOST-NAME or throws some kind of condition.
 HOST-NAME may also be an IP address in dotted quad notation or some other
@@ -423,7 +422,13 @@ SB-SYS:MAKE-FD-STREAM."))
   (let ((fd (socket-file-descriptor socket)))
     (unless (eql fd -1) ; already closed
       (cond ((slot-boundp socket 'stream)
-	     (close (slot-value socket 'stream)) ;; closes fd indirectly
+             (let ((stream (slot-value socket 'stream)))
+               #+threads
+               (close (two-way-stream-input-stream stream))
+               #+threads
+               (close (two-way-stream-output-stream stream))
+               #-threads
+               (close stream)) ;; closes fd indirectly
 	     (slot-makunbound socket 'stream))
 	    ((= (socket-close-low-level socket) -1)
 	     (socket-error "close")))
@@ -1154,9 +1159,12 @@ also known as unix-domain sockets."))
 ;;; the sockets will be closed upon garbage collection)
 ;;;
 
-(defun make-stream-from-fd (fd mode buffering &optional (name "FD-STREAM"))
+(defun dup (fd)
+  (ffi:c-inline (fd) (:int) :int "dup(#0)" :one-liner t))
+
+(defun make-stream-from-fd (fd mode &key buffering external-format (name "FD-STREAM"))
   (assert (stringp name) (name) "name must be a string.")
-  (c-inline (name fd (ecase mode
+  (let* ((smm-mode (ecase mode
 		       (:input (c-constant "smm_input"))
 		       (:output (c-constant "smm_output"))
 		       (:input-output (c-constant "smm_io"))
@@ -1166,40 +1174,84 @@ also known as unix-domain sockets."))
 		       (:output-wsock (c-constant "smm_output_wsock"))
 		       #+:wsock
 		       (:input-output-wsock (c-constant "smm_io_wsock"))
-		       )
-		  buffering)
-	    (t :int :int :object)
-	    t
-	    "si_set_buffering_mode(ecl_make_stream_from_fd(#0,#1,(enum ecl_smmode)#2,8,ECL_STREAM_DEFAULT_FORMAT,Cnil), #3)"
-	    :one-liner t))
+		       ))
+         (stream (ffi:c-inline (name fd smm-mode) (t :int :int) t
+                               "ecl_make_stream_from_fd(#0,#1,(enum ecl_smmode)#2,8,
+                                                        ECL_STREAM_DEFAULT_FORMAT,Cnil)"
+                               :one-liner t)))
+    (when buffering
+      (si::set-buffering-mode stream buffering))
+    (unless (eq external-format :default)
+      (setf (stream-external-format stream) external-format))
+    stream))
 
-(defmethod socket-make-stream ((socket socket)  &rest args &key (buffering-mode NIL))
-  (declare (ignore args))
+(defun auto-close-two-way-stream (stream)
+  (declare (si::c-local))
+  (ffi:c-inline (stream) (t) :void
+                "(#0)->stream.flags |= ECL_STREAM_CLOSE_COMPONENTS"
+                :one-liner t))
+
+(defun socket-make-stream-inner (fd input output buffering external-format)
+  ;; In Unix we have to create one stream per channel. The reason is
+  ;; that buffered I/O is done using ANSI C FILEs which do not support
+  ;; concurrent reads and writes -- if one thread is listening to the
+  ;; FILE it blocks all output.  The solution is to create a
+  ;; two-way-stream when both input and output are T, and force that
+  ;; stream to close its components (small hack in ECL). In Windows we
+  ;; do not have this problem because we do not know how to wrap a
+  ;; FILE around a socket.
+  (cond ((and input output)
+         #+wsock
+         (make-stream-from-fd fd :input-output-wsock
+                              :buffering buffering :external-format external-format)
+         #-wsock
+         (let* ((in (socket-make-stream-inner (dup fd) t nil buffering external-format))
+                (out (socket-make-stream-inner fd nil t buffering external-format))
+                (stream (make-two-way-stream in out)))
+           (auto-close-two-way-stream stream)
+           stream))
+        (input
+         (make-stream-from-fd fd #-wsock :input #+wsock :input-wsock
+                              :buffering buffering :external-format external-format))
+	(output
+	 (make-stream-from-fd fd #-wsock :output #+wsock :output-wsock
+                              :buffering buffering :external-format external-format))
+	(t
+	 (error "SOCKET-MAKE-STREAM: at least one of :INPUT or :OUTPUT has to be true."))))
+
+(defmethod socket-make-stream ((socket socket)
+			       &key (input nil input-p) (output nil output-p)
+			       (buffering :full) (external-format :default))
   (let ((stream (and (slot-boundp socket 'stream)
 		     (slot-value socket 'stream))))
     (unless stream
-      (setf stream (let ((fd (socket-file-descriptor socket)))
-		     (make-stream-from-fd fd #-:wsock :input-output #+:wsock :input-output-wsock
-					  buffering-mode)))
+      ;; Complicated default logic for compatibility with previous releases
+      ;; should disappear soon. (FIXME!)
+      (unless (or input-p output-p)
+	(setf input t output t))
+      (setf stream (socket-make-stream-inner (socket-file-descriptor socket)
+					     input output buffering external-format))
       (setf (slot-value socket 'stream) stream)
       #+ ignore
       (sb-ext:cancel-finalization socket))
     stream))
 
+(defmethod ext::stream-fd ((socket socket))
+  (socket-file-descriptor socket))
+
 #+:wsock
-(defmethod socket-make-stream ((socket named-pipe-socket) &rest args &key (buffering-mode NIL))
-  (declare (ignore args))
+(defmethod socket-make-stream ((socket named-pipe-socket)
+			       &key input output
+                               (buffering :full) (external-format :default))
   (let ((stream (and (slot-boundp socket 'stream)
 		     (slot-value socket 'stream))))
     (unless stream
       (setf stream
 	    (let* ((fd (socket-file-descriptor socket))
-		   (in (make-stream-from-fd fd :smm-input buffering-mode))
-		   (out (make-stream-from-fd fd :smm-output buffering-mode)))
+		   (in (make-stream-from-fd fd :smm-input buffering external-format))
+		   (out (make-stream-from-fd fd :smm-output buffering external-format)))
 	      (make-two-way-stream in out)))
-      (setf (slot-value socket 'stream) stream)
-      #+ ignore
-      (sb-ext:cancel-finalization socket))
+      (setf (slot-value socket 'stream) stream))
     stream))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1253,7 +1305,7 @@ also known as unix-domain sockets."))
   (:documentation "Common base class of socket related conditions."))
 
 (defmacro define-socket-condition (symbol name)
-  `(progn
+  `(let () ; Prevents evaluation of constant value at compilation time
      (defconstant ,symbol (c-constant ,(symbol-name symbol)))
      (define-condition ,name (socket-error)
        ((symbol :reader socket-error-symbol :initform (quote ,symbol))))
@@ -1335,7 +1387,7 @@ GET-NAME-SERVICE-ERRNO")
 		       (get-name-service-error-message num))))))
 
 (defmacro define-name-service-condition (symbol name)
-  `(progn
+  `(let ()
      (defconstant ,symbol (c-constant ,(symbol-name symbol)))
      (define-condition ,name (name-service-error)
        ((symbol :reader name-service-error-symbol :initform (quote ,symbol))))
@@ -1374,14 +1426,14 @@ GET-NAME-SERVICE-ERRNO")
 ;;; SOCKET OPTIONS
 ;;;
 
-(defun get-sockopt-int (fd const)
-  (let ((ret (c-inline (fd const) (:int :int) t
+(defun get-sockopt-int (fd level const)
+  (let ((ret (c-inline (fd level const) (:int :int :int) t
 "{
         int sockopt, ret;
         socklen_t socklen = sizeof(int);
 
 	ecl_disable_interrupts();
-	ret = getsockopt(#0,SOL_SOCKET,#1,&sockopt,&socklen);
+	ret = getsockopt(#0,#1,#2,&sockopt,&socklen);
 	ecl_enable_interrupts();
 
         @(return) = (ret == 0) ? ecl_make_integer(sockopt) : Cnil;
@@ -1390,37 +1442,36 @@ GET-NAME-SERVICE-ERRNO")
 	ret
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
-
-(defun get-sockopt-bool (fd const)
-  (let ((ret (c-inline (fd const) (:int :int) t
+(defun get-sockopt-bool (fd level const)
+  (let ((ret (c-inline (fd level const) (:int :int :int) t
 "{
         int sockopt, ret;
         socklen_t socklen = sizeof(int);
 
 	ecl_disable_interrupts();
-        ret = getsockopt(#0,SOL_SOCKET,#1,&sockopt,&socklen);
+        ret = getsockopt(#0,#1,#2,&sockopt,&socklen);
 	ecl_enable_interrupts();
 
-        @(return) = (ret == 0) ? Ct : Cnil;
+        @(return) = (ret == 0) ? ecl_make_integer(sockopt) : Cnil;
 }")))
     (if ret
-	ret
+	(/= ret 0)
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
 #+wsock
-(defun get-sockopt-timeval (fd const)
-  (* 1000 (get-sockopt-int fd const)))
+(defun get-sockopt-timeval (fd level const)
+  (* 1000 (get-sockopt-int fd level const)))
 
 #-wsock
-(defun get-sockopt-timeval (fd const)
-  (let ((ret (c-inline (fd const) (:int :int) t
+(defun get-sockopt-timeval (fd level const)
+  (let ((ret (c-inline (fd level const) (:int :int :int) t
 "{
 	struct timeval tv;
         socklen_t socklen = sizeof(struct timeval);
         int ret;
 
 	ecl_disable_interrupts();
-	ret = getsockopt(#0,SOL_SOCKET,#1,&tv,&socklen);
+	ret = getsockopt(#0,#1,#2,&tv,&socklen);
 	ecl_enable_interrupts();
 
         @(return) = (ret == 0) ? ecl_make_doublefloat((double)tv.tv_sec
@@ -1430,14 +1481,31 @@ GET-NAME-SERVICE-ERRNO")
 	ret
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
-(defun set-sockopt-int (fd const value)
-  (let ((ret (c-inline (fd const value) (:int :int :int) t
+(defun get-sockopt-linger (fd level const)
+  (let ((ret (c-inline (fd level const) (:int :int :int) t
 "{
-        int sockopt = #2;
+	struct linger sockopt;
+	socklen_t socklen = sizeof(struct linger);
+	int ret;
+
+	ecl_disable_interrupts();
+	ret = getsockopt(#0,#1,#2,&sockopt,&socklen);
+	ecl_enable_interrupts();
+
+	@(return) = (ret == 0) ? ecl_make_integer((sockopt.l_onoff != 0) ? sockopt.l_linger : 0) : Cnil;
+}")))
+    (if ret
+	ret
+	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
+
+(defun set-sockopt-int (fd level const value)
+  (let ((ret (c-inline (fd level const value) (:int :int :int :int) t
+"{
+        int sockopt = #3;
         int ret;
 
 	ecl_disable_interrupts();
-	ret = setsockopt(#0,SOL_SOCKET,#1,&sockopt,sizeof(int));
+	ret = setsockopt(#0,#1,#2,&sockopt,sizeof(int));
 	ecl_enable_interrupts();
 
         @(return) = (ret == 0) ? Ct : Cnil;
@@ -1446,34 +1514,34 @@ GET-NAME-SERVICE-ERRNO")
 	value
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
-(defun set-sockopt-bool (fd const value)
-  (let ((ret (c-inline (fd const value) (:int :int :object) t
+(defun set-sockopt-bool (fd level const value)
+  (let ((ret (c-inline (fd level const value) (:int :int :int :object) t
 "{
-        int sockopt = (#2 == Cnil) ? 0 : 1;
+        int sockopt = (#3 == Cnil) ? 0 : 1;
         int ret;
 
 	ecl_disable_interrupts();
-	ret = setsockopt(#0,SOL_SOCKET,#1,&sockopt,sizeof(int));
+	ret = setsockopt(#0,#1,#2,&sockopt,sizeof(int));
 	ecl_enable_interrupts();
 
         @(return) = (ret == 0) ? Ct : Cnil;
 }")))
     (if ret
-	ret
+	value
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
 #-wsock
-(defun set-sockopt-timeval (fd const value)
-  (let ((ret (c-inline (fd const value) (:int :int :double) t
+(defun set-sockopt-timeval (fd level const value)
+  (let ((ret (c-inline (fd level const value) (:int :int :int :double) t
 "{
 	struct timeval tv;
-	double tmp = #2;
+	double tmp = #3;
 	int ret;
 
 	ecl_disable_interrupts();
 	tv.tv_sec = (int)tmp;
 	tv.tv_usec = (int)((tmp-floor(tmp))*1000000.0);
-        ret = setsockopt(#0,SOL_SOCKET,#1,&tv,sizeof(struct timeval));
+        ret = setsockopt(#0,#1,#2,&tv,sizeof(struct timeval));
 	ecl_enable_interrupts();
 
         @(return) = (ret == 0) ? Ct : Cnil;
@@ -1483,39 +1551,64 @@ GET-NAME-SERVICE-ERRNO")
 	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
 #+wsock
-(defun set-sockopt-timeval (fd const value)
-  (set-sockopt-int fd const (* 1000 value)))
+(defun set-sockopt-timeval (fd level const value)
+  (set-sockopt-int fd level const (* 1000 value)))
+
+(defun set-sockopt-linger (fd level const value)
+  (let ((ret (c-inline (fd level const value) (:int :int :int :int) t
+"{
+	struct linger sockopt = {0, 0};
+	int value = #3;
+	int ret;
+
+	if (value > 0) {
+		sockopt.l_onoff = 1;
+		sockopt.l_linger = value;
+	}
+
+	ecl_disable_interrupts();
+	ret = setsockopt(#0,#1,#2,&sockopt,sizeof(struct linger));
+	ecl_enable_interrupts();
+
+	@(return) = (ret == 0) ? Ct : Cnil;
+}")))
+    (if ret
+	value
+	(error "Sockopt error: ~A" (c-inline () () :cstring "strerror(errno)" :one-liner t)))))
 
 (eval-when (:compile-toplevel :load-toplevel)
-  (defmacro define-sockopt (name c-const type &optional (read-only nil))
+  (defmacro define-sockopt (name c-level c-const type &optional (read-only nil))
     `(progn
        (export ',name)
        (defun ,name (socket)
 	 (,(intern (format nil "GET-SOCKOPT-~A" type))
 	   (socket-file-descriptor socket)
+	   (c-constant ,c-level)
 	   (c-constant ,c-const)))
        ,@(unless read-only
 	   `((defun (setf ,name) (value socket)
 	       (,(intern (format nil "SET-SOCKOPT-~A" type))
 		 (socket-file-descriptor socket)
+		 (c-constant ,c-level)
 		 (c-constant ,c-const)
 		 value)))))))
 
-(define-sockopt sockopt-type "SO_TYPE" int t)
-(define-sockopt sockopt-receive-buffer "SO_RCVBUF" int)
-(define-sockopt sockopt-receive-timeout "SO_RCVTIMEO" timeval)
-(define-sockopt sockopt-send-timeout "SO_SNDTIMEO" timeval)
-(define-sockopt sockopt-reuse-address "SO_REUSEADDR" bool)
-(define-sockopt sockopt-keep-alive "SO_KEEPALIVE" bool)
-(define-sockopt socket-dont-route "SO_DONTROUTE" bool)
-(define-sockopt socket-linger "SO_LINGER" bool)
+(define-sockopt sockopt-type "SOL_SOCKET" "SO_TYPE" int t)
+(define-sockopt sockopt-receive-buffer "SOL_SOCKET" "SO_RCVBUF" int)
+(define-sockopt sockopt-receive-timeout "SOL_SOCKET" "SO_RCVTIMEO" timeval)
+(define-sockopt sockopt-send-timeout "SOL_SOCKET" "SO_SNDTIMEO" timeval)
+(define-sockopt sockopt-reuse-address "SOL_SOCKET" "SO_REUSEADDR" bool)
+(define-sockopt sockopt-keep-alive "SOL_SOCKET" "SO_KEEPALIVE" bool)
+(define-sockopt sockopt-dont-route "SOL_SOCKET" "SO_DONTROUTE" bool)
+(define-sockopt sockopt-linger "SOL_SOCKET" "SO_LINGER" linger)
 
 #-(or :sun4sol2 :linux :wsock :cygwin)
-(define-sockopt sockopt-reuse-port "SO_REUSEPORT" bool)
+(define-sockopt sockopt-reuse-port "SOL_SOCKET" "SO_REUSEPORT" bool)
 
-(define-sockopt sockopt-tcp-nodelay "TCP_NODELAY" bool)
+(define-sockopt sockopt-tcp-nodelay "IPPROTO_TCP" "TCP_NODELAY" bool)
 
 ;; Add sockopts here as you need them...
 
 ;; Finished loading
 (provide 'sockets)
+(provide 'sb-bsd-sockets)
