@@ -108,6 +108,8 @@ char *function_c_name;
 char *required[MAXREQ];
 int nreq;
 
+int the_env_defined = 0;
+
 struct optional {
 	char *o_var;
 	char *o_init;
@@ -249,13 +251,26 @@ search_keyword(const char *name)
 }
 
 char *
-search_symbol(char *name, int *symbol_code)
+search_symbol(char *name, int *symbol_code, int code)
 {
 	int i;
 	for (i = 0; cl_symbols[i].name != NULL; i++) {
 		if (!strcasecmp(name, cl_symbols[i].name)) {
 			name = poolp;
-			if (i == 0) {
+                        if (code) {
+                                pushstr("MAKE_FIXNUM(/*");
+                                pushstr(cl_symbols[i].name);
+				pushstr("*/");
+				if (i >= 1000)
+					pushc((i / 1000) % 10 + '0');
+				if (i >= 100)
+					pushc((i / 100) % 10 + '0');
+				if (i >= 10)
+					pushc((i / 10) % 10 + '0');
+				pushc(i % 10 + '0');
+				pushstr(")");
+				pushc(0);
+                        } else if (i == 0) {
 				pushstr("Cnil");
 				pushc(0);
 			} else {
@@ -281,19 +296,20 @@ search_symbol(char *name, int *symbol_code)
 }
 
 char *
-read_symbol()
+read_symbol(int code)
 {
 	char c, *name = poolp;
+        char end = code? ']' : '\'';
 
 	c = readc();
-	while (c != '\'') {
+	while (c != end) {
 		if (c == '_') c = '-';
 		pushc(c); 
 		c = readc();
 	}
 	pushc(0);
 
-	name = search_symbol(poolp = name, 0);
+	name = search_symbol(poolp = name, 0, code);
 	if (name == NULL) {
 		name = poolp;
 		printf("\nUnknown symbol: %s\n", name);
@@ -385,7 +401,10 @@ read_token(void)
 		} else if (c == '@') {
 			c = readc();
 			if (c == '\'') {
-				(void)read_symbol();
+				(void)read_symbol(0);
+				poolp--;
+			} else if (c == '[') {
+				(void)read_symbol(1);
 				poolp--;
 			} else if (c == '@') {
 				pushc(c);
@@ -410,6 +429,7 @@ reset(void)
 {
 	int i;
 
+        the_env_defined = 0;
 	poolp = pool;
 	function = NULL;
 	function_symbol = "";
@@ -445,7 +465,7 @@ void
 get_function(void)
 {
 	function = read_function();
-	function_symbol = search_symbol(function, &function_code);
+	function_symbol = search_symbol(function, &function_code, 0);
 	if (function_symbol == NULL) {
 		function_symbol = poolp;
 		pushstr("Cnil");
@@ -467,7 +487,7 @@ get_lambda_list(void)
 			return;
 		if (c == '&') {
 			p = read_token();
-			goto OPTIONAL;
+			goto _OPT;
 		}
 		unreadc(c);
 		p = read_token();
@@ -476,15 +496,15 @@ get_lambda_list(void)
 		required[nreq++] = p;
 	}
 
-OPTIONAL:
+_OPT:
 	if (strcmp(p, "optional") != 0 && strcmp(p, "o") != 0)
-		goto REST;
+		goto _REST;
 	for (;;  nopt++) {
 		if ((c = nextc()) == ')')
 			return;
 		if (c == '&') {
 			p = read_token();
-			goto REST;
+			goto _REST;
 		}
 		if (nopt >= MAXOPT)
 			error("too many optional argument");
@@ -506,9 +526,9 @@ OPTIONAL:
 		}
 	}
 
-REST:
+_REST:
 	if (strcmp(p, "rest") != 0 && strcmp(p, "r") != 0)
-		goto KEY;
+		goto _KEY;
 	rest_flag = TRUE;
 	if ((c = nextc()) == ')' || c == '&')
 		error("&rest var missing");
@@ -519,11 +539,10 @@ REST:
 	if (c != '&')
 		error("& expected");
 	p = read_token();
-	goto KEY;
 
-KEY:
+_KEY:
 	if (strcmp(p, "key") != 0 && strcmp(p, "k") != 0)
-		goto AUX;
+		goto _AUX;
 	key_flag = TRUE;
 	for (;;  nkey++) {
 		if ((c = nextc()) == ')')
@@ -539,7 +558,7 @@ KEY:
 					error("& expected");
 				p = read_token();
 			}
-			goto AUX;
+			goto _AUX;
 		}
 		if (nkey >= MAXKEY)
 			error("too many optional argument");
@@ -576,7 +595,7 @@ KEY:
 		}
 	}
 
-AUX:
+_AUX:
 	if (strcmp(p, "aux") != 0 && strcmp(p, "a") != 0)
 		error("illegal lambda-list keyword");
 	for (;;) {
@@ -637,6 +656,7 @@ put_declaration(void)
   int simple_varargs;
 
   put_lineno();
+  the_env_defined = 1;
   fprintf(out, "\tconst cl_env_ptr the_env = ecl_process_env();\n");
   for (i = 0;  i < nopt;  i++) {
     put_lineno();
@@ -672,7 +692,9 @@ put_declaration(void)
   }
   if (nopt == 0 && !rest_flag && !key_flag) {
     put_lineno();
-    fprintf(out, "\tif (narg!=%d) FEwrong_num_arguments(MAKE_FIXNUM(%d));\n", nreq, function_code);
+    fprintf(out, "\tif (ecl_unlikely(narg!=%d))", nreq);
+    fprintf(out, "\t   FEwrong_num_arguments(MAKE_FIXNUM(%d));\n",
+            function_code);
   } else {
     simple_varargs = !rest_flag && !key_flag && ((nreq + nopt) < 32);
     if (key_flag) {
@@ -693,11 +715,11 @@ put_declaration(void)
 		rest_var, rest_var, ((nreq > 0) ? required[nreq-1] : "narg"),
 		nreq);
     put_lineno();
-    fprintf(out, "\tif (narg < %d", nreq);
+    fprintf(out, "\tif (ecl_unlikely(narg < %d", nreq);
     if (nopt > 0 && !rest_flag && !key_flag) {
       fprintf(out, "|| narg > %d", nreq + nopt);
     }
-    fprintf(out, ") FEwrong_num_arguments(MAKE_FIXNUM(%d));\n", function_code);
+    fprintf(out, ")) FEwrong_num_arguments(MAKE_FIXNUM(%d));\n", function_code);
     for (i = 0;  i < nopt;  i++) {
       put_lineno();
       fprintf(out, "\tif (narg > %d) {\n", nreq+i);
@@ -764,12 +786,15 @@ put_return(void)
 	int i, t;
 
 	t = tab_save+1;
+        
+        fprintf(out, "{\n");
+        if (!the_env_defined) {
+          put_tabs(t);
+          fprintf(out, "const cl_env_ptr the_env = ecl_process_env();\n");
+        }
 	if (nres == 0) {
-	  fprintf(out, "return0();");
-	} else if (nres == 1) {
-	  fprintf(out, "return1(%s);", result[0]);
+          fprintf(out, "the_env->nvalues = 0; return Cnil;\n");
 	} else {
-	  fprintf(out, "{\n");
 	  put_tabs(t);
 	  for (i = 0;  i < nres;  i++) {
 		put_tabs(t);
@@ -783,9 +808,9 @@ put_return(void)
 	  }
 	  put_tabs(t);
 	  fprintf(out, "return __value0;\n");
-	  put_tabs(tab_save);
-	  fprintf(out, "}\n");
 	}
+        put_tabs(tab_save);
+        fprintf(out, "}\n");
 }
 
 int
@@ -825,7 +850,14 @@ LOOP:
 	} else if (c == '\'') {
 		char *p;
 		poolp = pool;
-		p = read_symbol();
+		p = read_symbol(0);
+		pushc('\0');
+		fprintf(out,"%s",p);
+		goto LOOP;
+	}  else if (c == '[') {
+		char *p;
+		poolp = pool;
+		p = read_symbol(1);
 		pushc('\0');
 		fprintf(out,"%s",p);
 		goto LOOP;
