@@ -1053,7 +1053,7 @@ cl_symbols[] = {
 {SYS_ "*IGNORE-EOF-ON-TERMINAL-IO*",NULL},
 {SYS_ "*INDENT-FORMATTED-OUTPUT*",NULL},
 {SYS_ "*INIT-FUNCTION-PREFIX*",NULL},
-{SYS_ "*INTERRUPT-ENABLE*",NULL},
+{SYS_ "*INTERRUPTS-ENABLED*",NULL},
 {SYS_ "*KEEP-DEFINITIONS*",NULL},
 {SYS_ "*LOAD-HOOKS*",NULL},
 {SYS_ "*LOAD-SEARCH-LIST*",NULL},
@@ -1101,6 +1101,7 @@ cl_symbols[] = {
 {SYS_ "FILE-COLUMN","si_file_column"},
 {SYS_ "FILE-KIND","si_file_kind"},
 {SYS_ "FILL-POINTER-SET","si_fill_pointer_set"},
+{SYS_ "FILE-STREAM-FD","si_file_stream_fd"},
 {SYS_ "FIXNUMP","si_fixnump"},
 {SYS_ "FORMAT-ERROR",NULL},
 #ifdef ECL_CMU_FORMAT
@@ -1121,6 +1122,7 @@ cl_symbols[] = {
 {SYS_ "GETPID","si_getpid"},
 {SYS_ "HASH-SET","si_hash_set"},
 {SYS_ "HASH-TABLE-ITERATOR","si_hash_table_iterator"},
+{SYS_ "IHS-BDS","si_ihs_bds"},
 {SYS_ "IHS-ENV","si_ihs_env"},
 {SYS_ "IHS-FUN","si_ihs_fun"},
 {SYS_ "IHS-NEXT","si_ihs_next"},
@@ -1185,7 +1187,7 @@ cl_symbols[] = {
 {SYS_ "STANDARD-READTABLE","si_standard_readtable"},
 {SYS_ "STEPPER","OBJNULL"},
 {SYS_ "BASE-STRING-CONCATENATE","si_base_string_concatenate"},
-{SYS_ "STRING-TO-OBJECT","si_string_to_object"},
+{SYS_ "STRING-TO-OBJECT","ECL_NAME(si_string_to_object)"},
 {SYS_ "STRUCTURE-NAME","si_structure_name"},
 {SYS_ "STRUCTURE-PRINT-FUNCTION",NULL},
 {SYS_ "STRUCTURE-REF","si_structure_ref"},
@@ -1490,11 +1492,15 @@ cl_symbols[] = {
 {MP_ "PROCESS-PRESET","mp_process_preset"},
 {MP_ "PROCESS-RUN-FUNCTION","mp_process_run_function"},
 {MP_ "PROCESS-WHOSTATE","mp_process_whostate"},
+{MP_ "PROCESS-JOIN","mp_process_join"},
 {MP_ "MAKE-LOCK","mp_make_lock"},
 {KEY_ "RECURSIVE",NULL},
 {MP_ "RECURSIVE-LOCK-P","mp_recursive_lock_p"},
 {MP_ "LOCK-NAME","mp_lock_name"},
 {MP_ "LOCK-HOLDER","mp_lock_holder"},
+{MP_ "LOCK-COUNT","mp_lock_count"},
+{MP_ "LOCK-MINE-P","mp_lock_mine_p"},
+{MP_ "LOCK-COUNT-MINE","mp_lock_count_mine"},
 {MP_ "GET-LOCK","mp_get_lock"},
 {MP_ "GIVEUP-LOCK","mp_giveup_lock"},
 {MP_ "MAKE-CONDITION-VARIABLE","mp_make_condition_variable"},
@@ -1506,7 +1512,20 @@ cl_symbols[] = {
 {MP_ "INTERRUPT-PROCESS","mp_interrupt_process"},
 {MP_ "+LOAD-COMPILE-LOCK+",NULL},
 {MP_ "WITH-LOCK",NULL},
-{MP_ "WITHOUT-INTERRUPTS",NULL},
+{MP_ "BLOCK-SIGNALS","mp_block_signals"},
+{MP_ "RESTORE-SIGNALS","mp_restore_signals"},
+{MP_ "PROCESS-SUSPEND","mp_process_suspend"},
+{MP_ "PROCESS-RESUME","mp_process_resume"},
+{MP_ "SUSPEND-LOOP","mp_suspend_loop"},
+{MP_ "BREAK-SUSPEND-LOOP","mp_break_suspend_loop"},
+# ifdef ECL_SEMAPHORES
+{MP_ "SEMAPHORE",NULL},
+{MP_ "MAKE-SEMAPHORE","mp_make_semaphore"},
+{MP_ "SEMAPHORE-WAIT","mp_semaphore_wait"},
+{MP_ "SEMAPHORE-TRYWAIT","mp_semaphore_trywait"},
+{MP_ "SEMAPHORE-SIGNAL","mp_semaphore_signal"},
+{MP_ "SEMAPHORE-CLOSE","mp_semaphore_close"},
+# endif
 #endif
 
 {SYS_ "WHILE",NULL},
@@ -1515,7 +1534,9 @@ cl_symbols[] = {
 {SYS_ "*EXIT-HOOKS*",NULL},
 
 #ifdef CLOS
+{CLOS_ ".COMBINED-METHOD-ARGS.",NULL},
 {CLOS_ "*BUILTIN-CLASSES*",NULL},
+{CLOS_ "*NEXT-METHODS*",NULL},
 {CLOS_ "*OPTIMIZE-SLOT-ACCESS*",NULL},
 {CLOS_ "ACCESSOR-METHOD-SLOT-DEFINITION",NULL},
 {CLOS_ "ADD-DEPENDENT",NULL},
@@ -1616,7 +1637,7 @@ cl_symbols[] = {
 
 {SYS_ "*CODE-WALKER*",NULL},
 
-#ifdef ECL_DYNAMIC_FFI
+#if defined(HAVE_LIBFFI) || defined(ECL_DYNAMIC_FFI)
 {SYS_ "CALL-CFUN","si_call_cfun"},
 {KEY_ "CALLBACK",NULL},
 {SYS_ "MAKE-DYNAMIC-CALLBACK","si_make_dynamic_callback"},
@@ -1629,8 +1650,11 @@ cl_symbols[] = {
 {SYS_ "*ACTION-ON-UNDEFINED-VARIABLE*",NULL},
 
 {SYS_ "SET-BUFFERING-MODE","si_set_buffering_mode"},
+{KEY_ "NONE",NULL},
 {KEY_ "LINE-BUFFERED",NULL},
 {KEY_ "FULLY-BUFFERED",NULL},
+{KEY_ "LINE",NULL},
+{KEY_ "FULL",NULL},
 
 {SYS_ "CMP-ENV-REGISTER-MACROLET",NULL},
 
@@ -1757,6 +1781,7 @@ cl_symbols[] = {
 
 {EXT_ "LOAD-ENCODING",NULL},
 {EXT_ "MAKE-ENCODING",NULL},
+{EXT_ "ALL-ENCODINGS",NULL},
 
 {KEY_ "US-ASCII",NULL},
 {KEY_ "LITTLE-ENDIAN",NULL},
@@ -1785,5 +1810,76 @@ cl_symbols[] = {
 
 {SYS_ "READ-OBJECT-OR-IGNORE","si_read_object_or_ignore"},
 
+{EXT_ "OUTPUT-FLOAT-NAN",NULL},
+{EXT_ "OUTPUT-FLOAT-INFINITY",NULL},
+
+{EXT_ "READTABLE-LOCK","si_readtable_lock"},
+
+{SYS_ "+IO-SYNTAX-PROGV-LIST+",NULL},
+{SYS_ "+ECL-SYNTAX-PROGV-LIST+",NULL},
+{SYS_ "WITH-ECL-IO-SYNTAX",NULL},
+
+{SYS_ "STREAM-EXTERNAL-FORMAT-SET","si_stream_external_format_set"},
+
+{KEY_ "SYSV",NULL},
+{KEY_ "UNIX64",NULL},
+
+{EXT_ "WEAK-POINTER",NULL},
+{EXT_ "MAKE-WEAK-POINTER","si_make_weak_pointer"},
+{EXT_ "WEAK-POINTER-VALUE","si_weak_pointer_value"},
+
+{SYS_ "*ALLOW-WITH-INTERRUPTS*",NULL},
+
+{EXT_ "WITH-UNIQUE-NAMES",NULL},
+
+{SYS_ "HANDLE-SIGNAL","si_handle_signal"},
+
+{EXT_ "WITH-INTERRUPTS",NULL},
+{EXT_ "WITHOUT-INTERRUPTS",NULL},
+{EXT_ "WITH-LOCAL-INTERRUPTS",NULL},
+{EXT_ "ALLOW-WITH-INTERRUPTS",NULL},
+
+{SYS_ "*QUIT-TAG*",NULL},
+
+{EXT_ "ARRAY-RAW-DATA","si_array_raw_data"},
+
+{EXT_ "*PROGRAM-EXIT-CODE*",NULL},
+{EXT_ "EXIT","si_exit"},
+
+{EXT_ "ENVIRON","si_environ"},
+
+{KEY_ "ENVIRON",NULL},
+
+{EXT_ "MAKE-EXTERNAL-PROCESS",NULL},
+{EXT_ "EXTERNAL-PROCESS",NULL},
+{EXT_ "EXTERNAL-PROCESS-PID",NULL},
+{EXT_ "EXTERNAL-PROCESS-INPUT",NULL},
+{EXT_ "EXTERNAL-PROCESS-OUTPUT",NULL},
+{EXT_ "EXTERNAL-PROCESS-STATUS",NULL},
+
+{KEY_ "RUNNING",NULL},
+{KEY_ "EXITED",NULL},
+{KEY_ "SIGNALED",NULL},
+{KEY_ "STOPPED",NULL},
+
+{EXT_ "EXTERNAL-PROCESS-WAIT","si_external_process_wait"},
+
+#if defined(_MSC_VER) || defined(mingw32)
+{SYS_ "CLOSE-WINDOWS-HANDLE","si_close_windows_handle"},
+#endif
+
+{EXT_ "*INVOKE-DEBUGGER-HOOK*",NULL},
+
+{EXT_ "*DOCUMENTATION-POOL*",NULL},
+{EXT_ "ANNOTATE",NULL},
+{EXT_ "GET-ANNOTATION",NULL},
+{EXT_ "REMOVE-ANNOTATION",NULL},
+{EXT_ "FUNCTION-LAMBDA-LIST",NULL},
+{EXT_ "DEFAULT-ANNOTATION-LOGIC",NULL},
+{EXT_ "OPTIONAL-ANNOTATION",NULL},
+{EXT_ "LOCATION",NULL},
+
+{EXT_ "LISP-IMPLEMENTATION-VCS-ID",NULL},
+
 /* Tag for end of list */
 {NULL,NULL}};
